@@ -29,6 +29,7 @@ pub fn file_sink_from_fd(fd: i64) -> Result<fs::File> {
             use std::os::unix::io::FromRawFd;
             let fd = fd.try_into().context(
                 format!("Not a valid file descriptor: {}", fd))?;
+            check_fd(fd)?;
             Ok(unsafe {
                 fs::File::from_raw_fd(fd)
             })
@@ -46,6 +47,7 @@ pub fn source_from_fd(fd: i64) -> Result<fs::File> {
             use std::os::unix::io::FromRawFd;
             let fd = fd.try_into().context(
                 format!("Not a valid file descriptor: {}", fd))?;
+            check_fd(fd)?;
             Ok(unsafe {
                 fs::File::from_raw_fd(fd)
             })
@@ -56,6 +58,21 @@ pub fn source_from_fd(fd: i64) -> Result<fs::File> {
     }
 }
 
+/// Checks whether the given descriptor is valid.
+fn check_fd(fd: std::os::fd::RawFd) -> Result<()> {
+    unsafe {
+        // Try to dup(2) it.
+        let dup = libc::dup(fd);
+
+        if dup > 0 {
+            libc::close(dup);
+            Ok(())
+        } else {
+            Err(std::io::Error::last_os_error().into())
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests {
 }
