@@ -17,7 +17,6 @@ use openpgp::{
     Fingerprint,
     KeyHandle,
     cert::prelude::*,
-    crypto::hash::Digest,
     crypto::mpi::PublicKey,
     packet::{
         Key,
@@ -392,7 +391,7 @@ impl Record<'_> {
                     chrono::DateTime::<chrono::Utc>::from(t)
                 });
 
-                let mut uidhash = HashAlgorithm::RipeMD.context()?;
+                let mut uidhash = HashAlgorithm::RipeMD.context()?.for_digest();
                 uidhash.update(userid.value());
                 let uidhash = uidhash.into_digest()?;
 
