@@ -360,12 +360,12 @@ pub fn cmd_generate_revocation(config: &crate::Config, args: &[String])
 
     // Get the primary signer.  To that end, we need a valid cert
     // first to make password prompts more helpful for the user.
-    let null_policy = NullPolicy::new();
+    let null_policy = unsafe { NullPolicy::new() };
     let vcert = cert.with_policy(config.policy(), config.now())
         .or_else(|_| cert.with_policy(
-            config.policy(), cert.primary_key().creation_time()))
+            config.policy(), cert.primary_key().key().creation_time()))
         .or_else(|_| cert.with_policy(
-            &null_policy, cert.primary_key().creation_time()))
+            &null_policy, cert.primary_key().key().creation_time()))
         .context(format!("Key {:X} is not valid", cert.fingerprint()))?;
     // XXX: Would be nice to make this infallible, but
     // ipc::gnupg::KeyPair::with_cert takes a ValidCert, and there is
@@ -565,7 +565,7 @@ pub fn _print_mds(config: &crate::Config, algo: Option<HashAlgorithm>,
         let mut source = utils::open(config, &f)?;
 
         if let Some(algo) = algo {
-            let mut hash = algo.context()?;
+            let mut hash = algo.context()?.for_digest();
             std::io::copy(&mut source, &mut hash)?;
             let mut digest = vec![0; hash.digest_size()];
             hash.digest(&mut digest)?;
@@ -621,7 +621,7 @@ pub fn _print_mds(config: &crate::Config, algo: Option<HashAlgorithm>,
                 .chain((u8::from(SHA224) + 1..0xFF).into_iter())
                 .map(HashAlgorithm::from)
                 .filter(|a| a.is_supported())
-                .map(|h| h.context())
+                .map(|h| h.context().map(|c| c.for_digest()))
                 .collect::<Result<Vec<_>>>()?;
 
             let mut buf = vec![0; 4096];
@@ -639,13 +639,18 @@ pub fn _print_mds(config: &crate::Config, algo: Option<HashAlgorithm>,
                 hash.digest(&mut digest)?;
 
                 let mut offset = 0;
-                if f != "-" {
-                    safe_print!("{}:", f);
-                    offset += f.chars().count() + 1;
-                } else {
-                    if config.with_colons {
+                if config.with_colons {
+                    if f != "-" {
+                        safe_print!("{}:", f);
+                        offset += f.chars().count() + 1;
+                    } else {
                         safe_print!(":");
                     }
+                } else {
+                    if f != "-" {
+                        safe_print!("{}: ", f);
+                        offset += f.chars().count() + 2;
+                    }
                 }
 
                 if offset > INDENT_LIMIT {
