@@ -111,6 +111,7 @@ impl Policy for DeVSProducer {
                            -> openpgp::Result<()>
     {
         use SymmetricAlgorithm::*;
+        #[allow(deprecated)]
         match algo {
             AES128 | AES192 | AES256 | TripleDES => Ok(()),
             a => Err(Error::PolicyViolation(a.to_string(), None).into()),
@@ -161,7 +162,7 @@ impl Policy for DeVSConsumer {
            -> openpgp::Result<()>
     {
         use mpi::PublicKey::*;
-        match ka.mpis() {
+        match ka.key().mpis() {
             RSA { n, .. } => {
                 let l = n.bits();
                 if (l == 2048 || l == 3072 || l == 4096)
@@ -200,7 +201,7 @@ impl Policy for DeVSConsumer {
                         Err(Error::PolicyViolation(a.to_string(), None).into()),
                 }
             },
-            _ => return Err(Error::PolicyViolation(ka.fingerprint().to_string(),
+            _ => return Err(Error::PolicyViolation(ka.key().fingerprint().to_string(),
                                                    None).into()),
         }
     }
@@ -209,6 +210,7 @@ impl Policy for DeVSConsumer {
                            -> openpgp::Result<()>
     {
         use SymmetricAlgorithm::*;
+        #[allow(deprecated)]
         match algo {
             AES128 | AES192 | AES256 | TripleDES => Ok(()),
             Blowfish | Camellia128 | Camellia192 | Camellia256
