@@ -15,7 +15,6 @@ use crate::{
     common::{Common, Query},
     error_codes,
     status::Status,
-    utils,
 };
 
 /// Dispatches the --export-ssh-key command.
@@ -45,11 +44,7 @@ pub fn cmd_export_ssh_key(config: &mut crate::Config, args: &[String])
                     config.warn(format_args!("{}", e));
                     anyhow::anyhow!("export as ssh key failed: Unusable public key")
                 })?;
-            let mut sink = if let Some(name) = config.outfile() {
-                utils::create(config, name)?
-            } else {
-                Box::new(std::io::stdout())
-            };
+            let mut sink = config.sink()?;
             writeln!(&mut sink, "{}", key)?;
             Ok(())
         },
