@@ -310,7 +310,7 @@ async fn real_cmd_quick_add_key(config: &mut crate::Config<'_>, args: &[String])
         .insert_packets(vec![
             Packet::from(subkey.clone().role_into_subordinate()),
             binding.into(),
-        ])?;
+        ])?.0;
 
     // Actually store the cert.
     config.mut_keydb().update(
@@ -354,6 +354,7 @@ fn parse_key_parameter_part(_: &crate::Config, algo: &str, usage: &KeyFlags)
         PublicKeyAlgorithm::ECDH
     };
 
+    #[allow(deprecated)]
     match algo.to_ascii_lowercase().as_str() {
         "" | "-" | "default" | "rsa" =>
             Ok((PublicKeyAlgorithm::RSAEncryptSign, size.or(Some(3072)), None)),
@@ -522,6 +523,7 @@ impl Parameter {
             return Ok(PublicKeyAlgorithm::from(n));
         }
 
+        #[allow(deprecated)]
         match s.to_lowercase().as_str() {
             "default" => Ok(PublicKeyAlgorithm::RSAEncryptSign),
             "future-default" => Ok(if for_signing {
@@ -945,7 +947,7 @@ async fn create_key(config: &mut crate::Config<'_>, filename: &str, i: usize,
     }
 
     // Now add the new components and canonicalize once.
-    let cert = cert.insert_packets(acc)?;
+    let cert = cert.insert_packets(acc)?.0;
 
     // Build a revocation certificate.
     let revocation = CertRevocationBuilder::new()
@@ -976,7 +978,7 @@ async fn create_key(config: &mut crate::Config<'_>, filename: &str, i: usize,
     // Store the secrets in the agent.
     for subkey in cert.keys().secret() {
         agent.import(config.policy(),
-                     &cert, &subkey,
+                     &cert, subkey.key(),
                      config.batch, false).await?;
     }
 
@@ -1018,6 +1020,7 @@ fn do_create(config: &mut crate::Config<'_>,
                         Signature,
                         Option<Box<dyn Signer>>)>
 {
+    #[allow(deprecated)]
     let mut key: Key<key::SecretParts, key::UnspecifiedRole> = match algo {
         PublicKeyAlgorithm::RSAEncryptSign =>
             Key4::generate_rsa(bits.unwrap_or(3072))?,
@@ -1062,7 +1065,7 @@ fn do_create(config: &mut crate::Config<'_>,
             // GnuPG wants at least a 512-bit hash for P521 keys.
                 .set_hash_algo(HashAlgorithm::SHA512)
                 .sign_primary_key_binding(&mut subkey_signer,
-                                          &cert.primary_key(),
+                                          cert.primary_key().key(),
                                           &subkey)?;
             builder = builder.set_embedded_signature(backsig)?;
         }
@@ -1070,7 +1073,8 @@ fn do_create(config: &mut crate::Config<'_>,
         let sig = subkey.bind(primary_signer, &cert, builder)?;
 
         if let Some(password) = password {
-            subkey.secret_mut().encrypt_in_place(password)?;
+            let k = subkey.clone();
+            subkey.secret_mut().encrypt_in_place(&k, password)?;
         }
 
         Ok((subkey.role_into_unspecified(), sig, None))
@@ -1092,7 +1096,8 @@ fn do_create(config: &mut crate::Config<'_>,
             &mut signer, key.parts_as_public().role_as_primary())?;
 
         if let Some(password) = password {
-            key.secret_mut().encrypt_in_place(password)?;
+            let k = key.clone();
+            key.secret_mut().encrypt_in_place(&k, password)?;
         }
 
         Ok((key, sig, Some(Box::new(signer))))
@@ -1140,7 +1145,7 @@ fn add_primary_key_metadata(builder: SignatureBuilder,
     }
 
     if let Some(r) = revoker {
-        builder = builder.set_revocation_key(vec![r.clone()])?;
+        builder = builder.set_revocation_key(r.clone())?;
     }
 
     Ok(builder)
