@@ -3,7 +3,7 @@ use std::{
     collections::BTreeSet,
     fmt,
     fs,
-    io::{self, Read, Write},
+    io::{Read, Write},
     path::{Path, PathBuf},
     sync::{Arc, Mutex},
     time,
@@ -40,6 +40,8 @@ pub mod gnupg_interface;
 mod print;
 #[macro_use]
 mod macros;
+
+pub mod io;
 pub mod tracing;
 #[macro_use]
 pub mod argparse;
@@ -66,6 +68,7 @@ pub mod export;
 pub mod export_ssh_key;
 pub mod import;
 pub mod keyserver;
+use keyserver::KeyserverURL;
 pub mod sign;
 pub mod encrypt;
 pub mod list_keys;
@@ -123,7 +126,7 @@ pub struct Config<'store> {
     def_recipient_self: bool,
     def_secret_key: Vec<String>,
     def_sig_expire: Option<time::Duration>,
-    default_keyring: bool,
+    keyring_usage: KeyringUsage,
     dotlock_disable: bool,
     dry_run: bool,
     emit_version: usize,
@@ -255,7 +258,7 @@ impl<'store> Config<'store> {
             def_recipient_self: false,
             def_secret_key: vec![],
             def_sig_expire: None,
-            default_keyring: true,
+            keyring_usage: Default::default(),
             dotlock_disable: false,
             dry_run: false,
             emit_version: 0,
@@ -366,6 +369,28 @@ impl<'store> Config<'store> {
 
         agent.send_simple("RESET").await?;
 
+        if self.verbose > 0 && agent.is_restricted() {
+            self.warn(format_args!(
+                "connection to agent is in restricted mode"));
+        }
+
+        // If so, the options we can set are limited.
+        if ! agent.is_restricted() {
+            self.send_agent_options(&mut agent).await?;
+        }
+
+        // Options we can set on restricted connections.
+        agent.send_simple("OPTION agent-awareness=2.1.0").await?;
+
+        Ok(agent)
+    }
+
+    /// Customizes the connection to the GnuPG agent.
+    ///
+    /// This sets the options that we can only set on unrestricted
+    /// connections.
+    pub async fn send_agent_options(&self, agent: &mut gpg_agent::Agent)
+                                    -> Result<()> {
         if let Ok(tty) = std::env::var("GPG_TTY") {
             agent.send_simple(
                 format!("OPTION ttyname={}", tty)).await?;
@@ -410,11 +435,10 @@ impl<'store> Config<'store> {
                         dbus)).await?;
         }
         agent.send_simple("OPTION allow-pinentry-notify").await?;
-        agent.send_simple("OPTION agent-awareness=2.1.0").await?;
         agent.send_simple(format!("OPTION pinentry-mode={}",
                                   self.pinentry_mode.as_str())).await?;
 
-        Ok(agent)
+        Ok(())
     }
 
     /// Returns whether we're using the default home directory.
@@ -479,7 +503,10 @@ impl<'store> Config<'store> {
                             -> Result<Box<dyn openpgp::crypto::Signer + Send + Sync>>
     {
         let mut agent = self.connect_agent().await?;
-        agent.has_key(subkey).await?;
+
+        if ! agent.has_key(subkey).await? {
+            return Err(anyhow::anyhow!("No secret key"));
+        }
 
         let mut pair = agent.keypair(subkey)?.with_cert(vcert);
 
@@ -564,7 +591,7 @@ impl<'store> Config<'store> {
                 // the same, mostly because this search operation is
                 // so expensive.
                 let trust_root =
-                    self.keydb().get_certd_overlay()
+                    self.keydb().certd_overlay()
                     .and_then(|o| Ok(o.trust_root()?.fingerprint()))
                     .ok();
                 for cert in self.keydb().certs()
@@ -607,56 +634,50 @@ impl<'store> Config<'store> {
         }
     }
 
-    /// Returns certs matching a given query using groups and the
-    /// configured trust model.
+    /// Returns certs matching a given query using the configured
+    /// trust model.
+    ///
+    /// This does not expand groups.
     pub fn lookup_certs(&self, query: &Query) -> Result<Vec<(Validity, Arc<LazyCert<'store>>)>> {
-        self.lookup_certs_with(
-            self.trust_model_impl.with_policy(self, Some(self.now()))?.as_ref(),
-            query, true)
+        self.trust_model_impl.with_policy(self, Some(self.now()))?
+            .lookup(query)
     }
 
-    /// Returns certs matching a given query using groups and the
-    /// given trust model.
-    pub fn lookup_certs_with<'a>(&self,
-                                 vtm: &dyn trust::ModelViewAt<'a, 'store>,
-                                 query: &Query,
-                                 expand_groups: bool)
-        -> Result<Vec<(Validity, Arc<LazyCert<'store>>)>>
+    /// Returns recipient certs matching a given query using groups
+    /// and the configured trust model.
+    ///
+    /// If `query` refers to a group, returns a set of certificates
+    /// for every expansion of the group.
+    ///
+    /// If `query` does not refer to a group, returns a vector with a
+    /// single element consisting of the set of certificates returned
+    /// by the trust model.
+    pub fn lookup_recipients(&self, query: &Query)
+                             -> Result<Vec<Vec<(Validity,
+                                                Arc<LazyCert<'store>>)>>>
     {
+        let vtm = self.trust_model_impl.with_policy(self, Some(self.now()))?;
+
         match query {
             Query::Key(_) | Query::ExactKey(_) =>
-                (), // Let the trust model do the lookup.
+                Ok(vec![vtm.lookup(query)?]),
 
-            // Try to map using groups if `expand_groups` is true.  We
-            // don't want to lookup expanded names again, as we may
-            // walk into loops.  GnuPG also doesn't do that.
-            Query::Email(e) | Query::ExactUserID(e) => if expand_groups {
+            // Try to map using groups.  We don't want to lookup
+            // expanded names again, as we may walk into loops.  GnuPG
+            // also doesn't do that.
+            Query::Email(e) | Query::ExactUserID(e)
+                | Query::UserIDFragment(e) =>
                 if let Some(queries) = self.groups.get(e.as_str()) {
                     let mut acc = Vec::new();
                     for query in queries {
                         let q = query.parse()?;
-                        acc.append(
-                            &mut self.lookup_certs_with(vtm, &q, false)?);
+                        acc.push(vtm.lookup(&q)?);
                     }
-                    return Ok(acc);
-                }
-            },
-            // Maybe expand groups.  See comment above.
-            Query::UserIDFragment(f) => if expand_groups {
-                if let Some(queries) = self.groups.get(&f[..]) {
-                    let mut acc = Vec::new();
-                    for query in queries {
-                        let q = query.parse()?;
-                        acc.append(
-                            &mut self.lookup_certs_with(vtm, &q, false)?);
-                    }
-                    return Ok(acc);
-                }
-            },
+                    Ok(acc)
+                } else {
+                    Ok(vec![vtm.lookup(query)?])
+                },
         }
-
-        // Then, use the trust model to lookup the cert.
-        vtm.lookup(query)
     }
 
     /// Like `.keydb().lookup_by_cert_or_subkey` but emits
@@ -790,6 +811,14 @@ impl<'store> common::Common<'store> for Config<'store> {
         self.outfile.as_ref()
     }
 
+    fn sink(&self) -> Result<Box<dyn std::io::Write + Send + Sync>> {
+        if let Some(name) = self.outfile() {
+            utils::create(self, name)
+        } else {
+            Ok(Box::new(io::stdout_fail_verbosely()))
+        }
+    }
+
     fn policy(&self) -> &GPGPolicy {
         &self.policy
     }
@@ -823,6 +852,25 @@ impl<'store> common::Common<'store> for Config<'store> {
     }
 }
 
+/// Models `--no-default-keyring` and `--no-keyring`.
+#[derive(Default, PartialEq, Eq)]
+enum KeyringUsage {
+    /// Use the default keyring.
+    #[default]
+    DefaultKeyring,
+
+    /// Use the default keyring unless at least one keyring is given
+    /// via `--keyring`.
+    ///
+    /// Implements `--no-default-keyring`.
+    NoDefaultKeyring,
+
+    /// Do not use any keyring, not even the ones explicitly specified.
+    ///
+    /// Implements `--no-keyring`.
+    NoKeyring,
+}
+
 #[derive(Default)]
 struct Flags {
     disable_signer_uid: bool,
@@ -848,33 +896,6 @@ impl URL {
     }
 }
 
-#[derive(Debug, Clone, PartialEq, Eq)]
-pub struct KeyserverURL {
-    url: String,
-}
-
-impl KeyserverURL {
-    pub fn url(&self) -> &str {
-        &self.url
-    }
-}
-
-impl Default for KeyserverURL {
-    fn default() -> Self {
-        "hkps://keys.openpgp.org".parse().unwrap()
-    }
-}
-
-impl std::str::FromStr for KeyserverURL {
-    type Err = anyhow::Error;
-
-    fn from_str(s: &str) -> std::result::Result<Self, Self::Err> {
-        Ok(Self {
-            url: s.into(), // XXX: parsing
-        })
-    }
-}
-
 #[derive(Clone)]
 enum RequestOrigin {
     Local,
@@ -1058,7 +1079,6 @@ impl AsRef<str> for Keyring {
 
 struct Recipient {
     name: String,
-    #[allow(dead_code)]
     hidden: bool,
     #[allow(dead_code)]
     config: bool,
@@ -1255,7 +1275,6 @@ fn real_main() -> anyhow::Result<()> {
     let mut debug_level = None;
     let mut logfile = None;
     let mut fpr_maybe_cmd = false;
-    let mut default_keyring = false;
     let mut eyes_only = false;
     let mut s2k_digest: Option<HashAlgorithm> = None;
     let mut s2k_cipher: Option<SymmetricAlgorithm> = None;
@@ -1326,7 +1345,10 @@ fn real_main() -> anyhow::Result<()> {
                 | aGPGConfTest =>
             {
                 set_cmd(&mut command, cmd)?;
-                opt.default_keyring = false;
+
+                // To speed up these commands, we disable the
+                // initialization of the certificate store.
+                opt.keyring_usage = KeyringUsage::NoKeyring;
             },
 
             aVersion
@@ -1625,10 +1647,10 @@ fn real_main() -> anyhow::Result<()> {
             },
 
 	    oNoDefKeyring => {
-                default_keyring = false;
+                opt.keyring_usage = KeyringUsage::NoDefaultKeyring;
             },
 	    oNoKeyring => {
-                default_keyring = false;
+                opt.keyring_usage = KeyringUsage::NoKeyring;
             },
 
 	    oNoGreeting => {
@@ -2263,6 +2285,9 @@ fn real_main() -> anyhow::Result<()> {
                     "Error parsing unknown option".into()
                 }
             }
+        }).map_err(|e| {
+            let _ = opt.status().failure_from_error("option-parser", &e);
+            e
         })?;
 
         // We give the argument parsing code a chance to cleanly exit
@@ -2325,36 +2350,72 @@ fn real_main() -> anyhow::Result<()> {
 
     // XXX: More option frobbing.
 
-    // Get the default one if no keyring has been specified.
-    if keyrings.is_empty() {
-        opt.keydb.add_resource(&opt.homedir, "pubring.gpg", Some(true), false, true)?;
+    // Load keyrings unless the user opted out.
+    use sequoia_cert_store::store::openpgp_cert_d::CertD;
+    match opt.keyring_usage {
+        KeyringUsage::DefaultKeyring | KeyringUsage::NoDefaultKeyring => {
+            // Get the default one unless the user opted out.
+            if opt.keyring_usage == KeyringUsage::DefaultKeyring
+                || keyrings.is_empty()
+            {
+                opt.keydb.add_resource(&opt.homedir, "pubring.gpg",
+                                       Some(true), false, true)?;
 
-        // Use the keybox database if it is there.
-        let _ = opt.keydb.add_resource(
-            &opt.homedir, "gnupg-kbx-db:public-keys.d/pubring.db",
-            Some(true), true, false)?;
+                // Use the keybox database if it is there.
+                let _ = opt.keydb.add_resource(
+                    &opt.homedir, "gnupg-kbx-db:public-keys.d/pubring.db",
+                    Some(true), true, false)?;
+
+                // Add the certd overlay.
+                if opt.homedir_is_default()? {
+                    // If we're using the default GNUPGHOME, we use
+                    // the default openpgp-cert-d so that certificates
+                    // are shared.
+                    opt.keydb.add_certd_overlay(
+                        &CertD::default_store_path()?)?;
+                } else {
+                    // Otherwise, we create a openpgp-cert-d in the
+                    // GNUPGHOME.
+                    opt.keydb.add_certd_overlay(
+                        &opt.homedir().join("pubring.cert.d"))?;
+                }
+            }
+
+            for path in keyrings {
+                opt.keydb.add_resource(&opt.homedir, path.as_ref(),
+                                       None, true, false)?;
+            }
+        },
+
+        KeyringUsage::NoKeyring => (),
     }
 
-    for path in keyrings {
-        opt.keydb.add_resource(&opt.homedir, path.as_ref(), None, true, false)?;
+    // Adding the default keyring above implicitly adds a metadata
+    // overlay.  But, the user may have opted out.  Make sure we have
+    // a metadata overlay.
+    if opt.keydb.metadata_overlay().is_err() {
+        // We still need the certd overlay for the metadata.
+        if opt.homedir_is_default()? {
+            // If we're using the default GNUPGHOME, we use the default
+            // openpgp-cert-d so that certificates are shared.
+            opt.keydb.add_metadata_overlay(
+                &CertD::default_store_path()?)?;
+        } else {
+            // Otherwise, we create a openpgp-cert-d in the GNUPGHOME.
+            opt.keydb.add_metadata_overlay(
+                &opt.homedir().join("pubring.cert.d"))?;
+        }
     }
 
     if let Some(aGPGConfTest) = command {
         return Ok(());
     }
 
-    if opt.homedir_is_default()? {
-        // If we're using the default GNUPGHOME, we use the default
-        // openpgp-cert-d so that certificates are shared.
-        // XXX: Use CertD::default_location() once that is public.
-        opt.keydb.add_certd_overlay(
-            &dirs::data_dir().ok_or(anyhow::anyhow!("unsupported platform"))?
-                .join("pgp.cert.d"))?;
-    } else {
-        // Otherwise, we create a openpgp-cert-d in the GNUPGHOME.
-        opt.keydb.add_certd_overlay(&opt.homedir().join("pubring.cert.d"))?;
+    // Likewise, parcimonie should not be started if the user opted
+    // out of the default keyring.
+    if opt.keydb.certd_overlay().is_ok() {
+        parcimonie::start(&opt, command);
     }
-    parcimonie::start(&opt, command);
 
     // If a commad is likely to access at least the number of
     // certificates divided by the number of CPUs, then we should
@@ -2368,16 +2429,17 @@ fn real_main() -> anyhow::Result<()> {
 
     // Read the owner-trusts from our DB.
     // XXX: Currently, this is a plain text file.
-    let overlay = opt.keydb.get_certd_overlay()?;
-    let ownertrust_overlay =
-        overlay.path().join("_sequoia_gpg_chameleon_ownertrust");
-    if let Ok(mut f) = fs::File::open(ownertrust_overlay) {
-        // Suppress info messages while importing the ownertrust from
-        // our simple store.
-        let quiet = opt.quiet;
-        opt.quiet = true;
-        opt.trustdb.import_ownertrust(&opt, &mut f)?;
-        opt.quiet = quiet;
+    if let Ok(overlay) = opt.keydb.metadata_overlay() {
+        let ownertrust_overlay =
+            overlay.base_dir().join("_sequoia_gpg_chameleon_ownertrust");
+        if let Ok(mut f) = fs::File::open(ownertrust_overlay) {
+            // Suppress info messages while importing the ownertrust from
+            // our simple store.
+            let quiet = opt.quiet;
+            opt.quiet = true;
+            opt.trustdb.import_ownertrust(&opt, &mut f)?;
+            opt.quiet = quiet;
+        }
     }
 
     if let gpg_agent::PinentryMode::Loopback = opt.pinentry_mode {
@@ -2416,7 +2478,7 @@ fn real_main() -> anyhow::Result<()> {
         // descriptor to avoid closing the stream.  We may still want
         // to read data from this stream later.
         use std::os::unix::io::IntoRawFd;
-        pwfd.into_raw_fd();
+        let _ = pwfd.into_raw_fd();
     }
 
     // Read dirmngr's configuration.  We honor some of the options
@@ -2463,33 +2525,67 @@ fn real_main() -> anyhow::Result<()> {
         _ => (),
     }
 
+    // Which location to report when emitting failure status messages.
+    let mut location = None;
+
     let result = match command {
-        Some(aVerify) => if multifile {
-            verify::cmd_verify_files(&opt, &args)
-        } else {
-            verify::cmd_verify(&opt, &args)
+        Some(aVerify) => {
+            location = Some("verify");
+            if multifile {
+                verify::cmd_verify_files(&opt, &args)
+            } else {
+                verify::cmd_verify(&opt, &args)
+            }
         },
         Some(aDecrypt) => if multifile {
             decrypt::cmd_decrypt_files(&opt, &args)
         } else {
+            location = Some("decrypt");
             decrypt::cmd_decrypt(&opt, &args)
         },
-        Some(aExport) => export::cmd_export(&mut opt, &args, false),
-        Some(aExportSshKey) =>
-            export_ssh_key::cmd_export_ssh_key(&mut opt, &args),
+        Some(aExport) => {
+            location = Some("export");
+            export::cmd_export(&mut opt, &args, false)
+        },
+        Some(aExportSshKey) => {
+            location = Some("export-ssh-key");
+            export_ssh_key::cmd_export_ssh_key(&mut opt, &args)
+        },
         Some(aImport) => import::cmd_import(&mut opt, &args),
-        Some(aSign) => sign::cmd_sign(&mut opt, &args, detached_sig, false),
-        Some(aClearsign) => sign::cmd_sign(&mut opt, &args, detached_sig, true),
+        Some(aSign) => {
+            location = Some("sign");
+            sign::cmd_sign(&mut opt, &args, detached_sig, false)
+        },
+        Some(aClearsign) => {
+            location = Some("sign");
+            sign::cmd_sign(&mut opt, &args, detached_sig, true)
+        },
         Some(aEncr) => if multifile {
             encrypt::cmd_encrypt_files(&mut opt, &args)
         } else {
+            location = Some("encrypt");
             encrypt::cmd_encrypt(&mut opt, &args, false, false)
         },
-        Some(aSym) => encrypt::cmd_encrypt(&mut opt, &args, true, false),
-        Some(aSignSym) => encrypt::cmd_encrypt(&mut opt, &args, true, true),
-        Some(aEncrSym) => encrypt::cmd_encrypt(&mut opt, &args, true, false),
-        Some(aSignEncr) => encrypt::cmd_encrypt(&mut opt, &args, false, true),
-        Some(aSignEncrSym) => encrypt::cmd_encrypt(&mut opt, &args, true, true),
+        Some(aSym) => {
+            location = Some("symencrypt");
+            encrypt::cmd_encrypt(&mut opt, &args, true, false)
+        },
+        Some(aSignSym) => {
+            location = Some("sign-symencrypt");
+            encrypt::cmd_encrypt(&mut opt, &args, true, true)
+        },
+        Some(aEncrSym) => {
+            location = Some("encrypt");
+            encrypt::cmd_encrypt(&mut opt, &args, true, false)
+        },
+        Some(aSignEncr) => {
+            location = Some("sign-encrypt");
+            encrypt::cmd_encrypt(&mut opt, &args, false, true)
+        },
+        Some(aSignEncrSym) => {
+            location = Some("sign-encrypt");
+            encrypt::cmd_encrypt(&mut opt, &args, true, true)
+        },
         Some(aListKeys) =>
             list_keys::cmd_list_keys(&mut opt, &args, false),
         Some(aListSigs) => {
@@ -2512,13 +2608,32 @@ fn real_main() -> anyhow::Result<()> {
             commands::cmd_export_ownertrust(&opt, &args),
         Some(aListConfig) => commands::cmd_list_config(&opt, &args),
         Some(aGenRevoke) => commands::cmd_generate_revocation(&opt, &args),
-        Some(aEnArmor) => commands::cmd_enarmor(&opt, &args),
-        Some(aDeArmor) => commands::cmd_dearmor(&opt, &args),
-        Some(aRecvKeys) => keyserver::cmd_receive_keys(&mut opt, &args),
-        Some(aRefreshKeys) => keyserver::cmd_refresh_keys(&mut opt, &args),
+        Some(aEnArmor) => {
+            location = Some("enarmor");
+            commands::cmd_enarmor(&opt, &args)
+        },
+        Some(aDeArmor) => {
+            location = Some("dearmor");
+            commands::cmd_dearmor(&opt, &args)
+        },
+        Some(aRecvKeys) => {
+            location = Some("recv-keys");
+            keyserver::cmd_receive_keys(&mut opt, &args)
+        },
+        Some(aRefreshKeys) => {
+            location = Some("refresh-keys");
+            keyserver::cmd_refresh_keys(&mut opt, &args)
+        },
+        Some(aSendKeys) => {
+            location = Some("send-keys");
+            keyserver::cmd_send_keys(&mut opt, &args)
+        },
         Some(aPrintMD) => commands::print_md(&opt, &args),
         Some(aPrintMDs) => commands::print_mds(&opt, &args),
-        Some(aListPackets) => list_packets::cmd_list_packets(&opt, &args),
+        Some(aListPackets) => {
+            location = Some("-");
+            list_packets::cmd_list_packets(&opt, &args)
+        },
         Some(aKeygen) => generate_key::cmd_generate_key(&mut opt, &args, false),
         Some(aFullKeygen) => generate_key::cmd_generate_key(&mut opt, &args, true),
         Some(aQuickKeygen) =>
@@ -2526,7 +2641,10 @@ fn real_main() -> anyhow::Result<()> {
         Some(aQuickAddKey) => generate_key::cmd_quick_add_key(&mut opt, &args),
         Some(aQuickAddUid) => quick::cmd_quick_add_uid(&mut opt, &args),
         Some(aQuickRevUid) => quick::cmd_quick_revoke_uid(&mut opt, &args),
-        None => commands::cmd_implicit(&opt, &args),
+        None => {
+            location = Some("-");
+            commands::cmd_implicit(&opt, &args)
+        },
 
         // Our own extensions.
         Some(aXSequoiaParcimonie) =>
@@ -2556,14 +2674,40 @@ fn real_main() -> anyhow::Result<()> {
     match result {
         Ok(()) => {
             if let Some(c) = opt.override_status_code.get() {
+                if ! gnupg_interface::STRICT_OUTPUT {
+                    // If there hasn't been an explicit failure
+                    // message, emit a general one now.  GnuPG doesn't
+                    // do this, but given how fragile GPGME seems to
+                    // be, I think we should.
+                    let _ = opt.status().failure(
+                        "gpg-exit", error_codes::Error::GPG_ERR_GENERAL);
+                }
+
                 std::process::exit(c);
             }
             if opt.fail.get() {
+                if ! gnupg_interface::STRICT_OUTPUT {
+                    let _ = opt.status().failure(
+                        "gpg-exit", error_codes::Error::GPG_ERR_GENERAL);
+                }
                 std::process::exit(2);
             }
             Ok(())
         },
         Err(e) => {
+            // See if we have location information for this error.
+            // Note: if a more specific error has been already
+            // emitted, that's fine.
+            if let Some(location) = location {
+                let _ = opt.status().failure_from_error(location, &e);
+            }
+
+            // If there hasn't been an explicit failure message, emit
+            // a general one now.  That mirrors what GnuPG does in
+            // `g10_exit`.
+            let _ = opt.status().failure(
+                "gpg-exit", error_codes::Error::GPG_ERR_GENERAL);
+
             with_invocation_log(|w| write_error_chain_into(w, &e));
             if let Some(c) = opt.override_status_code.get() {
                 std::process::exit(c);
