@@ -2,7 +2,6 @@ use std::{
     cell::RefCell,
     fmt,
     fs,
-    io,
     path::{Path, PathBuf},
     sync::{Arc, Mutex},
     time::SystemTime,
@@ -21,6 +20,9 @@ mod print;
 mod macros;
 #[macro_use]
 pub mod argparse;
+
+pub mod io;
+
 use argparse::Argument;
 pub mod babel;
 pub mod clock;
@@ -141,6 +143,14 @@ impl<'store> common::Common<'store> for Config<'store> {
         self.outfile.as_ref()
     }
 
+    fn sink(&self) -> Result<Box<dyn std::io::Write + Send + Sync>> {
+        if let Some(name) = self.outfile() {
+            utils::create(self, name)
+        } else {
+            Ok(Box::new(io::stdout_fail_verbosely()))
+        }
+    }
+
     fn policy(&self) -> &GPGPolicy {
         &self.policy
     }
@@ -294,12 +304,32 @@ fn real_main() -> anyhow::Result<()> {
     match verify::cmd_verify(&opt, &args) {
         Ok(()) => {
             if opt.fail.get() {
+                // If there hasn't been an explicit failure message,
+                // emit a general one now.  GnuPG doesn't do this if
+                // no error bubbled up, but given how fragile GPGME
+                // seems to be, I think we should.
+                //
+                // Let's see if we can get away with adding some
+                // suspenders here.
+                let _ = opt.status_fd.failure(
+                    "verify", error_codes::Error::GPG_ERR_GENERAL);
+
                 std::process::exit(2);
             }
             Ok(())
         },
-        Err(e) if opt.verbose > 0 => Err(e),
-        Err(_) => std::process::exit(1),
+        Err(e) => {
+            // If there hasn't been an explicit failure message, emit
+            // a general one now.  That mirrors what GnuPG does in
+            // `g10_exit`, but gpgv-gpgv does not do this.
+            let _ = opt.status_fd.failure_from_error("verify", &e);
+
+            if opt.verbose > 0 {
+                Err(e)
+            } else {
+                std::process::exit(1)
+            }
+        },
     }
 }
 
