@@ -336,7 +336,7 @@ pub async fn do_import_cert(config: &mut crate::Config<'_>,
             // Clone stats so that we can summarize the changes.
             let s_before = s.clone();
             let (merged, changed) = existing.clone().insert_packets_merge(
-                cert.into_packets2(),
+                cert.into_packets(),
                 |old, new| {
                     match (&old, &new) {
                         (None, Packet::UserID(_))
@@ -481,7 +481,7 @@ pub async fn do_import_cert(config: &mut crate::Config<'_>,
         for subkey in key.to_cert()?.keys().secret() {
             // See if we import a new key or subkey.
             match agent.import(config.policy(),
-                               key.to_cert()?, &subkey,
+                               key.to_cert()?, subkey.key(),
                                config.batch, false).await
             {
                 Ok(_) => changed |= true,
@@ -523,7 +523,7 @@ pub async fn do_import_failed(config: &mut crate::Config<'_>,
     use crate::status::*;
 
     match e.downcast_ref::<openpgp::Error>() {
-        Some(openpgp::Error::UnsupportedCert2(_, _)) => {
+        Some(openpgp::Error::UnsupportedCert(_, _)) => {
             s.skipped_v3_keys += 1; // XXX: not very sharp
             return Ok(());
         },
