@@ -18,7 +18,6 @@ use sequoia_openpgp as openpgp;
 use openpgp::{
     Fingerprint,
     cert::raw::{RawCert, RawCertParser},
-    crypto::hash::Digest,
     KeyHandle,
     packet::UserID,
     parse::Parse,
@@ -642,7 +641,8 @@ impl<'store> Overlay<'store> {
 
     fn mtime_cache_path(&self, of: &Resource) -> PathBuf {
         let mut hash = HashAlgorithm::SHA256.context()
-            .expect("MTI hash algorithm");
+            .expect("MTI hash algorithm")
+            .for_digest();
         hash.update(of.path.to_string_lossy().as_bytes());
 
         let name = format!(
