@@ -471,7 +471,7 @@ where
         // while the agent does not yet have it.  Nevertheless, we
         // want to list the secrets.
         for skb in cert.to_cert()?.keys().secret() {
-            has_secret.insert(skb.fingerprint());
+            has_secret.insert(skb.key().fingerprint());
         }
 
         let cert_is_tsk = ! has_secret.is_empty();
@@ -565,7 +565,7 @@ where
                 kf
             },
             token_sn: token_sn.clone(),
-            compliance: cert.primary_key().compliance(config),
+            compliance: cert.primary_key().key().compliance(config),
         }.emit(config, &mut sink)?;
 
         for r in cert.revocation_keys(config.policy()) {
@@ -590,7 +590,7 @@ where
         if config.with_keygrip
             || (config.with_colons && (list_secret_keys_mode || have_secret))
         {
-            if let Ok(grip) = Keygrip::of(cert.primary_key().mpis()) {
+            if let Ok(grip) = Keygrip::of(cert.primary_key().key().mpis()) {
                 Record::Keygrip(grip).emit(config, &mut sink)?;
             }
         }
@@ -702,7 +702,7 @@ where
             }
 
             let vsubkey = subkey.clone().with_policy(p, now).ok();
-            let subkey_fp = subkey.fingerprint();
+            let subkey_fp = subkey.key().fingerprint();
             let have_secret = has_secret.contains(&subkey_fp);
             let token_sn = secrets.lookup_by_key(subkey.key()).await
                 .and_then(|i| if let Some(sn) = i.serialno() {
@@ -732,7 +732,7 @@ where
                     .and_then(|v| v.key_flags())
                     .unwrap_or_else(|| KeyFlags::empty()),
                 token_sn: token_sn.clone(),
-                compliance: subkey.compliance(config),
+                compliance: subkey.key().compliance(config),
             }.emit(config, &mut sink)?;
 
             if config.with_colons || config.with_subkey_fingerprint {
@@ -750,7 +750,7 @@ where
                 || (config.with_colons &&
                     (list_secret_keys_mode || have_secret))
             {
-                if let Ok(grip) = Keygrip::of(subkey.mpis()) {
+                if let Ok(grip) = Keygrip::of(subkey.key().mpis()) {
                     Record::Keygrip(grip).emit(config, &mut sink)?;
                 }
             }
@@ -848,7 +848,7 @@ where
     // Compute the issuer certificates (best effort) primary user ID
     // for display.
     let issuer_uid = if config.check_sigs && s.signature_creation_time()
-        .map(|sct| sct < cert.primary_key().creation_time())
+        .map(|sct| sct < cert.primary_key().key().creation_time())
         .unwrap_or(false)
     {
         IssuerUserID::TimeConflict
@@ -864,13 +864,13 @@ where
 
     // Now compute the signature validity.
     let sig_validity = if config.check_sigs && s.signature_creation_time()
-        .map(|sct| sct < cert.primary_key().creation_time())
+        .map(|sct| sct < cert.primary_key().key().creation_time())
         .unwrap_or(false)
     {
         config.warn(format_args!(
             "public key {:X} is {} seconds newer than the signature",
             cert.keyid(),
-            (cert.primary_key().creation_time().duration_since(
+            (cert.primary_key().key().creation_time().duration_since(
                 s.signature_creation_time().unwrap()).unwrap().as_secs())));
         sig_stats.errors += 1;
         SignatureValidity::OtherError
