@@ -46,7 +46,11 @@ pub fn cmd_list_packets(config: &crate::Config, args: &[String])
 {
     let input =
         utils::open(config, args.get(0).map(|s| s.as_str()).unwrap_or("-"))?;
-    let mut output = io::stdout(); // XXX
+    // Packet lists are always written to stdout (except if
+    // `--list-packets` is used in conjunction with other commands, in
+    // which case GnuPG sends the packet list to stderr; we just don't
+    // do that).
+    let mut output = crate::io::stdout();
 
     let mut dumper = PacketDumper::new();
 
@@ -83,15 +87,15 @@ pub fn cmd_list_packets(config: &crate::Config, args: &[String])
                 in_container.push(pp.packet.tag());
                 vec![]
             },
-            Packet::SEIP(_) | Packet::AED(_) => {
+            Packet::SEIP(_) => {
                 in_container.push(pp.packet.tag());
                 helper.uses_mdc();
                 let r = helper.decrypt(&pkesks, &skesks, None,
-                                       |algo, sk| pp.decrypt(algo, sk).is_ok());
+                                       &mut |algo, sk| pp.decrypt(algo, sk).is_ok());
                 if r.is_ok() {
                     config.status().emit(Status::DecryptionOkay)?;
                     // For compatibility reasons we issue GOODMDC also
-                    // for AEAD messages.
+                    // for SEIPDv2 messages.
                     config.status().emit(Status::GoodMDC)?;
                 }
                 config.status().emit(Status::EndDecryption)?;
@@ -310,10 +314,16 @@ impl PacketDumper {
                             Image::Unknown(_, data) =>
                                 writeln!(output, " [unknown image of size {}]",
                                          data.len())?,
+                            _ =>
+                                writeln!(output, " [unknown image of size {}]",
+                                         image.serialized_len())?,
                         },
                         Ok(Subpacket::Unknown(_, data)) =>
                             writeln!(output, " [unknown attribute of size {}]",
                                      data.len())?,
+                        Ok(s) =>
+                            writeln!(output, " [unknown attribute of size {}]",
+                                     s.serialized_len())?,
                         Err(_) =>
                             writeln!(output, "[invalid image]")?,
                     }
@@ -342,7 +352,7 @@ impl PacketDumper {
                 writeln!(output, " version {}, algo {}, keyid {:X}",
                          p.version(),
                          u8::from(p.pk_algo()),
-                         p.recipient())?;
+                         p.recipient().unwrap_or(KeyID::wildcard().into()))?;
                 match p.esk() {
                     mpi::Ciphertext::RSA { c } =>
                         self.dump_mpis(output,
@@ -433,14 +443,6 @@ impl PacketDumper {
                 writeln!(output, " length=20")?;
             },
 
-            AED(a) => {
-                writeln!(output, "\tVersion: {}", a.version())?;
-                writeln!(output, "\tSymmetric algo: {}", a.symmetric_algo())?;
-                writeln!(output, "\tAEAD: {}", a.aead())?;
-                writeln!(output, "\tChunk size: {}", a.chunk_size())?;
-                writeln!(output, "\tIV: {}", hex::encode(a.iv()))?;
-            },
-
             // openpgp::Packet is non-exhaustive.
             _ => writeln!(output, " [invalid]")?,
         }
@@ -577,13 +579,9 @@ impl PacketDumper {
                              _ => '?',
                          },
                          fp)?,
-            PreferredAEADAlgorithms(c) =>
-                writeln!(output, "{}    AEAD preferences: {}", i,
-                       c.iter().map(|c| format!("{:?}", c))
-                       .collect::<Vec<String>>().join(", "))?,
             IntendedRecipient(_) =>
                 writeln!(output, " (?)")?,
-            AttestedCertifications(_) =>
+            ApprovedCertifications(_) =>
                 writeln!(output, " (?)")?,
 
             // SubpacketValue is non-exhaustive.
