@@ -90,7 +90,7 @@ pub fn cmd_sign(config: &crate::Config, args: &[String],
     let mut signer = Signer::with_template(
         message,
         signers.pop().expect("at least one"),
-        openpgp::packet::signature::SignatureBuilder::new(class))
+        openpgp::packet::signature::SignatureBuilder::new(class))?
         .creation_time(timestamp)
         .hash_algo(hash_algo)?;
 
@@ -103,7 +103,7 @@ pub fn cmd_sign(config: &crate::Config, args: &[String],
     }
 
     for additional_signer in signers {
-        signer = signer.add_signer(additional_signer);
+        signer = signer.add_signer(additional_signer)?;
     }
 
     let mut message = signer.build()?;
@@ -217,13 +217,13 @@ pub async fn get_signers(config: &crate::Config<'_>)
         for key in vcert.keys().for_signing()
             .alive()
             .revoked(false)
-            .filter(|k| config.policy.public_key_algorithm(k.pk_algo()).is_ok())
+            .filter(|k| config.policy.public_key_algorithm(k.key().pk_algo()).is_ok())
         {
-            if let Ok(signer) = config.get_signer(&vcert, &key).await {
-                candidates.push((key.creation_time(),
+            if let Ok(signer) = config.get_signer(&vcert, key.key()).await {
+                candidates.push((key.key().creation_time(),
                                  key.primary(),
                                  signer,
-                                 key.pk_algo(), key.fingerprint()));
+                                 key.key().pk_algo(), key.key().fingerprint()));
             }
         }
         if candidates.is_empty() {
