@@ -559,7 +559,9 @@ impl Status<'_> {
                 let pk_len = 0;
 
                 writeln!(w, "ENC_TO {:X} {} {}",
-                         pkesk.recipient(),
+                         KeyID::from(
+                             pkesk.recipient()
+                                 .unwrap_or_else(|| KeyID::wildcard().into())),
                          u8::from(pkesk.pk_algo()),
                          pk_len)?;
             },
