@@ -64,7 +64,7 @@ impl TrustModel {
 
         match model {
             Auto => unreachable!(),
-            SequoiaGnuPG | PGP | TofuPGP =>
+            SequoiaGnuPG | PGP | Classic | TofuPGP =>
                 WoT::new().with_sequoia_roots()
                 .with_gnupg_roots(trust_config.marginals_needed,
                                   trust_config.completes_needed)
@@ -77,7 +77,7 @@ impl TrustModel {
                                   trust_config.completes_needed)
                 .build(),
             Always => Ok(Box::new(always::Always::default())),
-            _ => Err(anyhow::anyhow!("Trust model {:?} not implemented", self))
+            _ => Err(anyhow::anyhow!("Trust model {:?} not implemented", model))
         }
     }
 }
