@@ -307,7 +307,7 @@ impl Record {
             }))),
 
             12 => Ok(Some(Record::Trust {
-                fingerprint: Fingerprint::from_bytes(&b[2..22]),
+                fingerprint: Fingerprint::from_bytes(4, &b[2..22])?,
                 ownertrust:     b[22].try_into()?,
                 depth:          b[23],
                 min_ownertrust: b[24],
