@@ -1,6 +1,6 @@
 use std::{
     collections::HashSet,
-    io,
+    io::{self, Write as _},
 };
 
 use anyhow::Result;
@@ -9,7 +9,6 @@ use sequoia_openpgp as openpgp;
 use openpgp::{
     Cert,
     KeyID,
-    crypto::hash::Digest,
     packet::{
         Packet,
         Signature,
@@ -441,7 +440,7 @@ impl<'a, 'store> VHelper<'a, 'store> {
 
     /// Computes the signature id, a hash over the signature.
     fn compute_signature_id(&self, sig: &Signature) -> Result<String> {
-        let mut h = HashAlgorithm::SHA1.context()?;
+        let mut h = HashAlgorithm::SHA1.context()?.for_digest();
 
         // Algorithms.
         h.write_all(&[
@@ -605,7 +604,7 @@ impl<'a, 'store> VHelper<'a, 'store> {
                         self.control.error(
                             format_args!("bad data signature from key {:X}: \
                                           Wrong key usage (0x{:02x}, 0x{:x})",
-                                         ka.map(|ka| ka.keyid())
+                                         ka.map(|ka| ka.key().keyid())
                                          .unwrap_or_else(|| KeyID::wildcard()),
                                          u8::from(sig.typ()),
                                          ka.and_then(|ka| ka.key_flags())
@@ -690,7 +689,7 @@ impl<'a, 'store> VHelper<'a, 'store> {
         match error {
             None => {
                 self.control.status().emit(Status::GoodSig {
-                    issuer: ka.fingerprint().into(),
+                    issuer: ka.key().fingerprint().into(),
                     primary_uid: primary_uid.as_bytes().to_vec().into(),
                 })?;
             },
@@ -700,13 +699,13 @@ impl<'a, 'store> VHelper<'a, 'store> {
                 })?;
 
                 self.control.status().emit(Status::ExpKeySig {
-                    issuer: ka.fingerprint().into(),
+                    issuer: ka.key().fingerprint().into(),
                     primary_uid: primary_uid.as_bytes().to_vec().into(),
                 })?;
             },
             Some(openpgp::Error::InvalidKey(_)) => {
                 self.control.status().emit(Status::RevKeySig {
-                    issuer: ka.fingerprint().into(),
+                    issuer: ka.key().fingerprint().into(),
                     primary_uid: primary_uid.as_bytes().to_vec().into(),
                 })?;
             },
@@ -723,7 +722,7 @@ impl<'a, 'store> VHelper<'a, 'store> {
                 "Good signature from {:?}", primary_uid));
         }
         for uid in ka.cert().userids() {
-            let uid = String::from_utf8_lossy(uid.value());
+            let uid = String::from_utf8_lossy(uid.userid().value());
             if uid != primary_uid {
                 self.control.warn(format_args!(
                     "                    {:?}", uid));
@@ -742,7 +741,7 @@ impl<'a, 'store> VHelper<'a, 'store> {
                 // First, the warning text, preferring the cert
                 // revocation.
                 if let RevocationStatus::Revoked(_) =
-                    ka.cert().revocation_status()
+                    ka.valid_cert().revocation_status()
                 {
                     self.control.warn(format_args!(
                         "WARNING: This key has been revoked by its owner!"));
@@ -770,7 +769,7 @@ impl<'a, 'store> VHelper<'a, 'store> {
                         }
                     }
                 } else if let RevocationStatus::Revoked(rev) =
-                    ka.cert().revocation_status()
+                    ka.valid_cert().revocation_status()
                 {
                     for sig in rev {
                         if let Some((rfr, msg)) = sig.reason_for_revocation() {
@@ -819,7 +818,7 @@ impl<'a, 'store> VHelper<'a, 'store> {
 
         // Cryptographically valid.
         self.control.status().emit(Status::ValidSig {
-            issuer: ka.fingerprint(),
+            issuer: ka.key().fingerprint(),
             creation_time: sig.signature_creation_time()
                 .expect("every well-formed signature has one"),
             expire_time: sig.signature_expiration_time(),
@@ -858,7 +857,7 @@ impl<'a, 'store> VHelper<'a, 'store> {
                         all_expired_or_revoked: false,
                     })?;
                 } else if let RevocationStatus::Revoked(_) =
-                    ka.cert().revocation_status()
+                    ka.valid_cert().revocation_status()
                 {
                     self.control.status().emit(Status::KeyRevoked)?;
 
@@ -926,7 +925,7 @@ impl<'a, 'store> VHelper<'a, 'store> {
         };
 
         if print_fingerprint || self.control.with_fingerprint() {
-            let fp = ka.fingerprint();
+            let fp = ka.key().fingerprint();
             let cert_fp = ka.cert().fingerprint();
             let primary = fp == cert_fp;
 
@@ -998,7 +997,7 @@ impl<'a, 'store> VHelper<'a, 'store> {
         let primary_uid = crate::utils::best_effort_primary_uid(
             self.control.policy(), ka.cert());
         self.control.status().emit(Status::BadSig {
-            issuer: ka.fingerprint().into(),
+            issuer: ka.key().fingerprint().into(),
             primary_uid: primary_uid.as_bytes().to_vec().into(),
         })?;
 
@@ -1020,7 +1019,7 @@ impl<'a, 'store> VHelper<'a, 'store> {
         for result in results {
             match result {
                 Ok(GoodChecksum { sig, ka, .. }) => {
-                    if self.emit_signature(sig, ka, None, ka.cert().cert(),
+                    if self.emit_signature(sig, ka, None, ka.cert(),
                                            false)?
                     {
                         continue;
@@ -1083,13 +1082,13 @@ impl<'a, 'store> VHelper<'a, 'store> {
                         None
                     };
 
-                    if self.emit_signature(sig, ka, e, ka.cert().cert(), false)?
+                    if self.emit_signature(sig, ka, e, ka.cert(), false)?
                     {
                         continue;
                     }
 
                     let openpgp_error = error.downcast_ref::<openpgp::Error>();
-                    if sig.verify(ka.key()).is_ok()
+                    if sig.verify_signature(ka.key()).is_ok()
                         && self.control.policy().signature(
                             &sig, HashAlgoSecurity::CollisionResistance).is_ok()
                     {
@@ -1108,7 +1107,7 @@ impl<'a, 'store> VHelper<'a, 'store> {
                     continue;
                 },
                 Err(BadSignature { sig, ka, error }) => {
-                    if self.emit_signature(sig, ka, None, ka.cert().cert(),
+                    if self.emit_signature(sig, ka, None, ka.cert(),
                                            false)?
                     {
                         continue;
@@ -1121,7 +1120,15 @@ impl<'a, 'store> VHelper<'a, 'store> {
                     }
 
                     continue;
-                }
+                },
+                Err(UnknownSignature { .. }) => {
+                    self.bad_checksums += 1;
+                    continue;
+                },
+                Err(_) => {
+                    self.bad_checksums += 1;
+                    continue;
+                },
             };
         }
 
@@ -1145,6 +1152,16 @@ impl<'a, 'store> VerificationHelper for VHelper<'a, 'store> {
                         filename: p.filename().map(|n| n.to_vec()),
                     })?;
 
+                // XXX: GnuPG processes CSF messages in a streaming
+                // fashion, whereas RFC9580 no longer supports
+                // streaming CSF messages, because human readable
+                // messages are short, so they can be buffered.
+                //
+                // As a result, GnuPG does not emit a PLAINTEXT_LENGTH
+                // status message, but we do.  We should suppress the
+                // message when processing a CSF message, but we don't
+                // know if we are indeed processing a CSF message, see
+                // https://gitlab.com/sequoia-pgp/sequoia/-/issues/1134
                 if let BodyLength::Full(l) = pp.header().length() {
                     // Subtract the Literal Data packet's header
                     // fields from the packet length.
@@ -1168,10 +1185,10 @@ impl<'a, 'store> VerificationHelper for VHelper<'a, 'store> {
            .flatten()
            .filter_map(|cert| cert.to_cert().ok().cloned())
            .filter(|c| self.control.policy.public_key_algorithm(
-               c.primary_key().pk_algo()).is_ok())
+               c.primary_key().key().pk_algo()).is_ok())
            .map(|c| c.retain_subkeys(
                |sk| self.control.policy.public_key_algorithm(
-                   sk.pk_algo()).is_ok()))
+                   sk.key().pk_algo()).is_ok()))
            .collect())
     }
 
