@@ -14,7 +14,7 @@ use openpgp::{
         Serialize,
         SerializeInto,
         stream::{
-            Message, Encryptor2 as Encryptor, LiteralWriter,
+            Message, Encryptor, LiteralWriter,
         },
     },
     types::SymmetricAlgorithm,
@@ -266,7 +266,8 @@ fn general_purpose(cs: CipherSuite) -> Result<()> {
     let cert = experiment.artifact(
         "cert",
         || CertBuilder::general_purpose(
-            cs, Some("Alice Lovelace <alice@lovelace.name>"))
+            Some("Alice Lovelace <alice@lovelace.name>"))
+            .set_cipher_suite(cs)
             .set_creation_time(Experiment::now())
             .generate()
             .map(|(cert, _rev)| cert),
@@ -284,7 +285,7 @@ fn restricted_agent() -> Result<()> {
     let cert = experiment.artifact(
         "cert",
         || CertBuilder::general_purpose(
-            None, Some("Alice Lovelace <alice@lovelace.name>"))
+            Some("Alice Lovelace <alice@lovelace.name>"))
             .set_creation_time(Experiment::now())
             .generate()
             .map(|(cert, _rev)| cert),
@@ -528,7 +529,7 @@ fn a_cert() -> Result<()> {
     let cert = experiment.artifact(
         "cert",
         || CertBuilder::general_purpose(
-            None, Some("Alice Lovelace <alice@lovelace.name>"))
+            Some("Alice Lovelace <alice@lovelace.name>"))
             .set_creation_time(Experiment::now())
             .generate()
             .map(|(cert, _rev)| cert),
