@@ -13,7 +13,6 @@ use openpgp::{
     serialize::{Serialize, SerializeInto, stream::*},
     types::*,
 };
-use openpgp::serialize::stream::Encryptor2 as Encryptor;
 
 use super::super::*;
 
@@ -73,6 +72,7 @@ fn literal() -> Result<()> {
 
     let p = experiment.artifact(
         "text", || {
+            #[allow(deprecated)]
             let mut l = Literal::new(DataFormat::Text);
             l.set_body(PLAINTEXT.into());
             Packet::from(l).to_vec()
@@ -274,7 +274,7 @@ fn signature() -> Result<()> {
                 Key4::<_, UnspecifiedRole>::generate_ecc(true, Curve::Ed25519)?
                 .into_keypair()?;
             let message = Message::new(&mut sink);
-            let mut signer = Signer::new(message, pair)
+            let mut signer = Signer::new(message, pair)?
                 .detached().build()?;
             signer.write_all(PLAINTEXT)?;
             signer.finalize()?;
@@ -292,7 +292,7 @@ fn signature() -> Result<()> {
                 .into_keypair()?;
             let message = Message::new(&mut sink);
             let mut signer = Signer::with_template(
-                message, pair, SignatureBuilder::new(SignatureType::Text))
+                message, pair, SignatureBuilder::new(SignatureType::Text))?
                 .detached().build()?;
             signer.write_all(PLAINTEXT)?;
             signer.finalize()?;
@@ -302,6 +302,7 @@ fn signature() -> Result<()> {
         |b| Ok(b.to_vec()))?;
     list_packets(&mut experiment, p)?;
 
+    #[allow(deprecated)]
     let p = experiment.artifact(
         "clowncar-sig", || {
             use openpgp::types::*;
@@ -317,8 +318,8 @@ fn signature() -> Result<()> {
             let message = Message::new(&mut sink);
             let h = HashAlgorithm::default();
             let t = SignatureBuilder::new(SignatureType::Binary)
-                .set_attested_certifications(
-                    vec![vec![b'@'; h.context()?.digest_size()]])?
+                .set_approved_certifications(
+                    vec![vec![b'@'; h.context()?.for_digest().digest_size()]])?
                 .set_exportable_certification(false)?
                 .set_features(Features::sequoia())?
                 .set_intended_recipients(
@@ -354,7 +355,7 @@ fn signature() -> Result<()> {
                     b"something bad happened \"'\\\r\n\t)")?
                 .set_regular_expression(b"o.O \"'\\\r\n\t)")?
                 .set_revocable(true)?
-                .set_revocation_key(vec![(&cert).into()])?
+                .set_revocation_key((&cert).into())?
                 .set_signature_target(
                     PublicKeyAlgorithm::DSA,
                     HashAlgorithm::SHA384,
@@ -369,7 +370,7 @@ fn signature() -> Result<()> {
                     }, false)?)?;
                     Ok(a)
                 })?;
-            let mut signer = Signer::with_template(message, pair, t)
+            let mut signer = Signer::with_template(message, pair, t)?
                 .detached().build()?;
             signer.write_all(PLAINTEXT)?;
             signer.finalize()?;
@@ -395,7 +396,7 @@ fn inline_signed() -> Result<()> {
             let pair =
                 Key4::<_, UnspecifiedRole>::generate_ecc(true, Curve::Ed25519)?
                 .into_keypair()?;
-            let message = Signer::new(message, pair)
+            let message = Signer::new(message, pair)?
                 .build()?;
             let mut message = LiteralWriter::new(message).build()?;
             message.write_all(PLAINTEXT)?;
@@ -457,12 +458,13 @@ fn encrypted() -> Result<()> {
 
     let p = experiment.artifact(
         "for-cert", || {
+            let p = sequoia_openpgp::policy::StandardPolicy::new();
             let mut buf = vec![];
             let message = Message::new(&mut buf);
             let message = Encryptor::with_session_key(
                 message, algo, sk.clone())?
                 .add_recipients(vec![
-                    key.keys().subkeys().nth(0).unwrap().key(),
+                    key.keys().with_policy(&p, None).subkeys().nth(0).unwrap(),
                 ])
                 .build()?;
             let mut message = LiteralWriter::new(message).build()?;
@@ -616,7 +618,8 @@ fn general_purpose(cs: CipherSuite) -> Result<()> {
     let cert = experiment.artifact(
         "cert",
         || CertBuilder::general_purpose(
-            cs, Some("Alice Lovelace <alice@lovelace.name>"))
+            Some("Alice Lovelace <alice@lovelace.name>"))
+            .set_cipher_suite(cs)
             .set_creation_time(Experiment::now())
             .generate()
             .and_then(|(cert, _rev)| cert.to_vec()),
@@ -627,7 +630,8 @@ fn general_purpose(cs: CipherSuite) -> Result<()> {
     let key = experiment.artifact(
         "key",
         || CertBuilder::general_purpose(
-            cs, Some("Alice Lovelace <alice@lovelace.name>"))
+            Some("Alice Lovelace <alice@lovelace.name>"))
+            .set_cipher_suite(cs)
             .set_creation_time(Experiment::now())
             .generate()
             .and_then(|(key, _rev)| key.as_tsk().to_vec()),
@@ -638,7 +642,8 @@ fn general_purpose(cs: CipherSuite) -> Result<()> {
     let key = experiment.artifact(
         "revocation",
         || CertBuilder::general_purpose(
-            cs, Some("Alice Lovelace <alice@lovelace.name>"))
+            Some("Alice Lovelace <alice@lovelace.name>"))
+            .set_cipher_suite(cs)
             .set_creation_time(Experiment::now())
             .generate()
             .and_then(|(_key, rev)| Packet::from(rev).to_vec()),
@@ -657,7 +662,6 @@ fn locked_key() -> Result<()> {
     let key = experiment.artifact(
         "key",
         || CertBuilder::general_purpose(
-            None,
             Some("Alice Lovelace <alice@lovelace.name>"))
             .set_creation_time(Experiment::now())
             .set_password(Some("password".into()))
