@@ -74,7 +74,8 @@ fn general_purpose(cs: CipherSuite) -> Result<()> {
     let cert = experiment.artifact(
         "cert",
         || CertBuilder::general_purpose(
-            cs, Some("Alice Lovelace <alice@lovelace.name>"))
+            Some("Alice Lovelace <alice@lovelace.name>"))
+            .set_cipher_suite(cs)
             .set_creation_time(Experiment::now())
             .generate()
             .map(|(cert, _rev)| cert),
@@ -108,7 +109,7 @@ fn restricted_agent() -> Result<()> {
     let cert = experiment.artifact(
         "cert",
         || CertBuilder::general_purpose(
-            None, Some("Alice Lovelace <alice@lovelace.name>"))
+            Some("Alice Lovelace <alice@lovelace.name>"))
             .set_creation_time(Experiment::now())
             .generate()
             .map(|(cert, _rev)| cert),
@@ -563,7 +564,7 @@ fn issue_114_a() -> Result<()> {
                 .set_key_flags(KeyFlags::empty().set_signing())?;
             let binding = subkey.bind(&mut keypair, &c, builder)?;
             let c = c.insert_packets(vec![Packet::from(subkey),
-                                          Packet::from(binding)])?;
+                                          Packet::from(binding)])?.0;
 
             // Now add a signing subkey that is not expired.
             let mut subkey: Key<_, key::SubordinateRole> =
@@ -577,7 +578,7 @@ fn issue_114_a() -> Result<()> {
                 .set_key_flags(KeyFlags::empty().set_signing())?;
             let binding = subkey.bind(&mut keypair, &c, builder)?;
             let c = c.insert_packets(vec![Packet::from(subkey),
-                                          Packet::from(binding)])?;
+                                          Packet::from(binding)])?.0;
             let filtered = c.as_tsk()
                 .set_filter(move |k| k.fingerprint() != newer_subkey_fp)
                 .to_vec()?;
