@@ -23,7 +23,7 @@ fn always() -> Result<()> {
     let cert = experiment.artifact(
         "cert",
         || CertBuilder::general_purpose(
-            None, Some("Alice Lovelace <alice@lovelace.name>"))
+            Some("Alice Lovelace <alice@lovelace.name>"))
             .set_creation_time(Experiment::now())
             .generate()
             .map(|(cert, _rev)| cert),
@@ -58,7 +58,7 @@ fn pgp() -> Result<()> {
         let cert = experiment.artifact(
             "cert",
             || CertBuilder::general_purpose(
-                None, Some("Alice Lovelace <alice@lovelace.name>"))
+                Some("Alice Lovelace <alice@lovelace.name>"))
                 .set_creation_time(Experiment::now())
                 .generate()
                 .map(|(cert, _rev)| cert),
@@ -109,7 +109,7 @@ fn auto() -> Result<()> {
         let cert = experiment.artifact(
             "cert",
             || CertBuilder::general_purpose(
-                None, Some("Alice Lovelace <alice@lovelace.name>"))
+                Some("Alice Lovelace <alice@lovelace.name>"))
                 .set_creation_time(Experiment::now())
                 .generate()
                 .map(|(cert, _rev)| cert),
