@@ -236,7 +236,7 @@ fn signature_types() -> Result<()> {
         Unknown(PersonaCertification.into()),
         Unknown(CasualCertification.into()),
         Unknown(PositiveCertification.into()),
-        Unknown(AttestationKey.into()),
+        Unknown(CertificationApproval.into()),
         Unknown(SubkeyBinding.into()),
         Unknown(PrimaryKeyBinding.into()),
         Unknown(DirectKey.into()),
@@ -913,7 +913,7 @@ fn revoked_cert() -> Result<()> {
                 .add_userid("Alice Lovelace <alice@lovelace.name>")
                 .add_signing_subkey()
                 .generate()?;
-            Ok(cert.insert_packets2(rev)?.0)
+            Ok(cert.insert_packets(rev)?.0)
         },
         |a, f| a.as_tsk().serialize(f),
         |b| Cert::from_bytes(&b))?;
@@ -966,7 +966,7 @@ fn revoked_subkey() -> Result<()> {
                                            b"don't use it anymore")?
                 .build(&mut signer, &cert, subkey.key(), None)?;
 
-            Ok(cert.insert_packets2(sig)?.0)
+            Ok(cert.insert_packets(sig)?.0)
         },
         |a, f| a.as_tsk().serialize(f),
         |b| Cert::from_bytes(&b))?;
@@ -1018,7 +1018,7 @@ fn revoked_cert_and_subkey() -> Result<()> {
                                            b"don't use it anymore")?
                 .build(&mut signer, &cert, subkey.key(), None)?;
 
-            Ok(cert.insert_packets2(vec![sig, rev])?.0)
+            Ok(cert.insert_packets(vec![sig, rev])?.0)
         },
         |a, f| a.as_tsk().serialize(f),
         |b| Cert::from_bytes(&b))?;
